/* mxCrypto -- C++ Wrapper for the many fast goodies defined in SSLeay.

   (c) Marc-Andre Lemburg; All Rights Reserved; mailto: mal@lemburg.com
   See the documentation for further copyright information or contact
   the author.

 */

/* Uncomment to enable debugging output when run with -d flag. See
   mxstdlib.h for details. */
/*#define MAL_DEBUG*/

#include "mx.h"
#include "mxCrypto.h"
#include <ctype.h>

/* --- Utilities ---------------------------------------------------------- */

PyObject *str2hex(PyObject *v) 
{
    PyObject *w = 0;
    int i,len;
    char *raw,*hex;
    static const char hexdigits[] = "0123456789abcdef";

    Py_Assert(PyString_Check(v),
	      PyExc_TypeError,
	      "str2hex() needs string argument");
    /* Convert to HEX */
    len = PyString_GET_SIZE(v);
    w = PyString_FromStringAndSize(NULL,2*len);
    if (!w)
	goto onError;
    raw = PyString_AS_STRING(v);
    hex = PyString_AS_STRING(w);
    for (i = 0; i < len; i ++) {
	unsigned char c = (unsigned char)*raw;
	
	*hex++ = hexdigits[c >> 4];
	*hex++ = hexdigits[c & 0x0F];
	raw++;
    }
    return w;

 onError:
    Py_XDECREF(w);
    return NULL;
}

PyObject *hex2str(PyObject *v) 
{
    PyObject *w = 0;
    int i,len;
    char *raw,*hex;
    static const char hexdigits[] = "0123456789abcdef";

    Py_Assert(PyString_Check(v),
	      PyExc_TypeError,
	      "hex2str() needs string argument");
    /* Convert to string */
    len = PyString_GET_SIZE(v);
    Py_Assert(len % 2 == 0,
	      PyExc_TypeError,
	      "need 2-digit hex string argument");
    len >>= 1;
    w = PyString_FromStringAndSize(NULL,len);
    if (!w)
	goto onError;
    hex = PyString_AS_STRING(v);
    raw = PyString_AS_STRING(w);
    for (i = 0; i < len; i++,raw++) {
	register char c;
	register int j;

	c = tolower(*hex++);
	for (j = 0; j < (int)sizeof(hexdigits); j++)
	  if (c == hexdigits[j]) {
	    *raw = j << 4;
	    break;
	  }
	if (j == sizeof(hexdigits)) {
	  DPRINTF("Failed: '%c' (%u) at %i\n",c,(unsigned int)c,i);
	  Py_Error(PyExc_ValueError,
		   "argument contains non-hex characters");
	}

	c = tolower(*hex++);
	for (j = 0; j < (int)sizeof(hexdigits); j++)
	  if (c == hexdigits[j]) {
	    *raw += j;
	    break;
	  }
	if (j == sizeof(hexdigits)) {
	  DPRINTF("Failed2: '%c' (%u) at %i\n",c,(unsigned int)c,i);
	  Py_Error(PyExc_ValueError,
		   "argument contains non-hex characters");
	}
    }
    return w;

 onError:
    Py_XDECREF(w);
    return NULL;
}

/* --- StreamSipher Implementation ---------------------------------------- */

StreamCipher::StreamCipher(PyStringObject *key,
			   int cipher_mode)
{
    DPRINTF("StreamCipher constructor called for 0x%lx\n",(long)this);
    
    mode = cipher_mode;
#if 0
    /* This doesn't work for some reason I have yet to find out.  The
       exception thrown is not catched by the code in the SWIG
       wrapper. */
    Py_Assert(PyString_GET_SIZE(key) > 0,
	      PyExc_ValueError,
	      "empty key");
#endif
    return;

 onError:
    throw mxSWIGError;
}

StreamCipher::~StreamCipher()
{
    DPRINTF("StreamCipher destructor called for 0x%lx\n",(long)this);
}

PyObject *StreamCipher::encrypt(PyStringObject *input)
{
    Py_Error(PyExc_SystemError,
	     "not supported");
 onError:
    return NULL;
}

PyObject *StreamCipher::decrypt(PyStringObject *input)
{
    Py_Error(PyExc_SystemError,
	     "not supported");
 onError:
    return NULL;
}

/* --- BlockCipher Implementation ------------------------------------ */

BlockCipher::BlockCipher(PyStringObject *key,
			 int cipher_mode,
			 PyStringObject *init)
    : StreamCipher(key,cipher_mode)
{
    DPRINTF("BlockCipher constructor called for 0x%lx\n",(long)this);
    
    state = 0;
    Py_Assert(PyString_GET_SIZE(key) > 0,
	      PyExc_ValueError,
	      "empty key");
    if (cipher_mode == CBC || cipher_mode == CFB) {
	Py_AssertWithArg(init && PyString_GET_SIZE(init) == blocksize,
			 PyExc_TypeError,
			 "need an init vector of length %i",blocksize);
	memcpy(&iv[0],PyString_AS_STRING(init),sizeof(iv));
    } else
	Py_Assert(cipher_mode == ECB,
		  PyExc_ValueError,
		  "unsupported mode");
    return;
    
 onError:
    throw mxSWIGError;
}

PyObject *BlockCipher::reinit(PyStringObject *init)
{
    Py_AssertWithArg(PyString_GET_SIZE(init) == blocksize,
		     PyExc_TypeError,
		     "need an init vector of length %i",blocksize);
    memcpy(&iv[0],PyString_AS_STRING(init),sizeof(iv));
    Py_ReturnNone();
    
 onError:
    return NULL;
}

/* --- RC2 Implementation ------------------------------------------- */

RC2Cipher::RC2Cipher(PyStringObject *key,
		     int cipher_mode,
		     PyStringObject *init)
    : BlockCipher(key,cipher_mode,init)
{
    RC2_set_key(&cipher_key,
	       (int)PyString_GET_SIZE(key),
	       (unsigned char *)PyString_AS_STRING(key),1024);
    return;
    
 onError:
    throw mxSWIGError;
}

PyObject *RC2Cipher::convert(PyStringObject *input,
			     int dir)
{
    PyObject *output;
    unsigned char *p = (unsigned char *)PyString_AS_STRING(input);
    unsigned char *q;
    long len = PyString_GET_SIZE(input);
    
    output = PyString_FromStringAndSize(NULL,PyString_GET_SIZE(input));
    if (!output)
	goto onError;
    q = (unsigned char *)PyString_AS_STRING(output);

    if (mode == ECB) {
	Py_Assert(len % blocksize == 0,
		  PyExc_ValueError,
		  "string length must be multiple of blocksize");
	while (len > 0) {
	    RC2_ecb_encrypt(p,q,
			    &cipher_key, dir);
	    p += blocksize;
	    q += blocksize;
	    len -= blocksize;
	}
    }
    else if (mode == CBC) {
	Py_Assert(len % blocksize == 0,
		  PyExc_ValueError,
		  "string length must be multiple of blocksize");
	RC2_cbc_encrypt(p,q,len,
			&cipher_key, iv, dir);
    }
    else if (mode == CFB) {
	RC2_cfb64_encrypt(p,q,len,
			  &cipher_key, iv, &state, dir);
    }
    else
	Py_Error(PyExc_ValueError,
		 "unsupported mode");
    
    return output;

 onError:
    return NULL;
}

PyObject *RC2Cipher::encrypt(PyStringObject *input)
{
    return convert(input,RC2_ENCRYPT);
}

PyObject *RC2Cipher::decrypt(PyStringObject *input)
{
    return convert(input,RC2_DECRYPT);
}

/* --- RC4 Implementation ------------------------------------------------ */

RC4Cipher::RC4Cipher(PyStringObject *key,
		     int cipher_mode)
    : StreamCipher(key,ECB)
{
    Py_Assert(PyString_GET_SIZE(key) > 0,
	      PyExc_ValueError,
	      "empty key");
    Py_Assert(cipher_mode == ECB,
	      PyExc_ValueError,
	      "RC4 only supports ECB mode");

    RC4_set_key(&cipher_key,
		(int)PyString_GET_SIZE(key),
		(unsigned char *)PyString_AS_STRING(key));
    return;
    
 onError:
    throw mxSWIGError;
}

PyObject *RC4Cipher::encrypt(PyStringObject *input)
{
    PyObject *output;
    
    output = PyString_FromStringAndSize(NULL,PyString_GET_SIZE(input));
    if (!output)
	goto onError;

    RC4(&cipher_key,
	(unsigned long)PyString_GET_SIZE(input),
	(unsigned char *)PyString_AS_STRING(input), 
	(unsigned char *)PyString_AS_STRING(output));
    return output;

 onError:
    return NULL;
}

PyObject *RC4Cipher::decrypt(PyStringObject *input)
{
    PyObject *output;
    
    output = PyString_FromStringAndSize(NULL,PyString_GET_SIZE(input));
    if (!output)
	goto onError;

    RC4(&cipher_key,
	(unsigned long)PyString_GET_SIZE(input),
	(unsigned char *)PyString_AS_STRING(input), 
	(unsigned char *)PyString_AS_STRING(output));
    return output;

 onError:
    return NULL;
}

/* --- RC5 Implementation ------------------------------------------- */

RC5Cipher::RC5Cipher(PyStringObject *key,
		     int cipher_mode,
		     PyStringObject *init,
		     int rounds)
    : BlockCipher(key,cipher_mode,init)
{
    RC5_32_set_key(&cipher_key,
		   (int)PyString_GET_SIZE(key),
		   (unsigned char *)PyString_AS_STRING(key),rounds);
    return;
    
 onError:
    throw mxSWIGError;
}

PyObject *RC5Cipher::convert(PyStringObject *input,
			     int dir)
{
    PyObject *output;
    unsigned char *p = (unsigned char *)PyString_AS_STRING(input);
    unsigned char *q;
    long len = PyString_GET_SIZE(input);
    
    output = PyString_FromStringAndSize(NULL,PyString_GET_SIZE(input));
    if (!output)
	goto onError;
    q = (unsigned char *)PyString_AS_STRING(output);

    if (mode == ECB) {
	Py_Assert(len % blocksize == 0,
		  PyExc_ValueError,
		  "string length must be multiple of blocksize");
	while (len > 0) {
	    RC5_32_ecb_encrypt(p,q,
			       &cipher_key, dir);
	    p += blocksize;
	    q += blocksize;
	    len -= blocksize;
	}
    }
    else if (mode == CBC) {
	Py_Assert(len % blocksize == 0,
		  PyExc_ValueError,
		  "string length must be multiple of blocksize");
	RC5_32_cbc_encrypt(p,q,len,
			   &cipher_key, iv, dir);
    }
    else if (mode == CFB) {
	RC5_32_cfb64_encrypt(p,q,len,
			     &cipher_key, iv, &state, dir);
    }
    else
	Py_Error(PyExc_ValueError,
		 "unsupported mode");
    
    return output;

 onError:
    return NULL;
}

PyObject *RC5Cipher::encrypt(PyStringObject *input)
{
    return convert(input,RC5_ENCRYPT);
}

PyObject *RC5Cipher::decrypt(PyStringObject *input)
{
    return convert(input,RC5_DECRYPT);
}

/* --- Blowfish Implementation ------------------------------------------- */

BlowfishCipher::BlowfishCipher(PyStringObject *key,
			       int cipher_mode,
			       PyStringObject *init)
    : BlockCipher(key,cipher_mode,init)
{
    BF_set_key(&cipher_key,
	       (int)PyString_GET_SIZE(key),
	       (unsigned char *)PyString_AS_STRING(key));
    return;
    
 onError:
    throw mxSWIGError;
}

PyObject *BlowfishCipher::convert(PyStringObject *input,
				  int dir)
{
    PyObject *output;
    unsigned char *p = (unsigned char *)PyString_AS_STRING(input);
    unsigned char *q;
    long len = PyString_GET_SIZE(input);
    
    output = PyString_FromStringAndSize(NULL,PyString_GET_SIZE(input));
    if (!output)
	goto onError;
    q = (unsigned char *)PyString_AS_STRING(output);

    if (mode == ECB) {
	Py_Assert(len % blocksize == 0,
		  PyExc_ValueError,
		  "string length must be multiple of blocksize");
	while (len > 0) {
	    BF_ecb_encrypt(p,q,
			   &cipher_key, dir);
	    p += blocksize;
	    q += blocksize;
	    len -= blocksize;
	}
    }
    else if (mode == CBC) {
	Py_Assert(len % blocksize == 0,
		  PyExc_ValueError,
		  "string length must be multiple of blocksize");
	BF_cbc_encrypt(p,q,len,
		       &cipher_key, iv, dir);
    }
    else if (mode == CFB) {
	BF_cfb64_encrypt(p,q,len,
			 &cipher_key, iv, &state, dir);
    }
    else
	Py_Error(PyExc_ValueError,
		 "unsupported mode");
    
    return output;

 onError:
    return NULL;
}

PyObject *BlowfishCipher::encrypt(PyStringObject *input)
{
    return convert(input,BF_ENCRYPT);
}

PyObject *BlowfishCipher::decrypt(PyStringObject *input)
{
    return convert(input,BF_DECRYPT);
}

/* --- IDEA Implementation ------------------------------------------- */

IDEACipher::IDEACipher(PyStringObject *key,
		       int cipher_mode,
		       PyStringObject *init)
    : BlockCipher(key,cipher_mode,init)
{
    Py_AssertWithArg(PyString_GET_SIZE(key) == keysize,
		     PyExc_ValueError,
		     "need %i byte key string for IDEA",keysize);
    idea_set_encrypt_key((unsigned char *)PyString_AS_STRING(key),
			 &encrypt_key);
    idea_set_decrypt_key(&encrypt_key,&decrypt_key);
    
    return;
    
 onError:
    throw mxSWIGError;
}

PyObject *IDEACipher::convert(PyStringObject *input,
			      int dir)
{
    PyObject *output;
    unsigned char *p = (unsigned char *)PyString_AS_STRING(input);
    unsigned char *q;
    long len = PyString_GET_SIZE(input);
    IDEA_KEY_SCHEDULE *cipher_key;

    if (dir == IDEA_ENCRYPT)
	cipher_key = &encrypt_key;
    else
	cipher_key = &decrypt_key;
    
    output = PyString_FromStringAndSize(NULL,PyString_GET_SIZE(input));
    if (!output)
	goto onError;
    q = (unsigned char *)PyString_AS_STRING(output);

    if (mode == ECB) {
	Py_Assert(len % blocksize == 0,
		  PyExc_ValueError,
		  "string length must be multiple of blocksize");
	while (len > 0) {
	    idea_ecb_encrypt(p,q,
			     cipher_key);
	    p += blocksize;
	    q += blocksize;
	    len -= blocksize;
	}
    }
    else if (mode == CBC) {
	Py_Assert(len % blocksize == 0,
		  PyExc_ValueError,
		  "string length must be multiple of blocksize");
	idea_cbc_encrypt(p,q,len,
			 cipher_key, iv, dir);
    }
    else if (mode == CFB) {
	/* See special notes in the SSLeay docs on this mode... */
	idea_cfb64_encrypt(p,q,len,
			   &encrypt_key, iv, &state, dir);
    }
    else
	Py_Error(PyExc_ValueError,
		 "unsupported mode");
    
    return output;

 onError:
    return NULL;
}

PyObject *IDEACipher::encrypt(PyStringObject *input)
{
    return convert(input,IDEA_ENCRYPT);
}

PyObject *IDEACipher::decrypt(PyStringObject *input)
{
    return convert(input,IDEA_DECRYPT);
}

/* --- DES Implementation ------------------------------------------- */

DESCipher::DESCipher(PyStringObject *key,
		       int cipher_mode,
		       PyStringObject *init)
    : BlockCipher(key,cipher_mode,init)
{
    Py_INCREF(key);
    key_string = key;
    
    Py_AssertWithArg(PyString_GET_SIZE(key) == keysize,
		     PyExc_ValueError,
		     "need %i byte key string for DES",keysize);
    des_set_key((des_cblock *)PyString_AS_STRING(key),
		cipher_key);
    return;
    
 onError:
    throw mxSWIGError;
}

DESCipher::~DESCipher()
{
    Py_XDECREF(key_string);
}

PyObject *DESCipher::convert(PyStringObject *input,
			      int dir)
{
    PyObject *output;
    des_cblock *p = (des_cblock *)PyString_AS_STRING(input);
    des_cblock *q;
    long len = PyString_GET_SIZE(input);

    output = PyString_FromStringAndSize(NULL,PyString_GET_SIZE(input));
    if (!output)
	goto onError;
    q = (des_cblock *)PyString_AS_STRING(output);

    if (mode == ECB) {
	Py_Assert(len % blocksize == 0,
		  PyExc_ValueError,
		  "string length must be multiple of blocksize");
	while (len > 0) {
	    des_ecb_encrypt(p,q,
			    cipher_key, dir);
	    p++;
	    q++;
	    len -= blocksize;
	}
    }
    else if (mode == CBC) {
	Py_Assert(len % blocksize == 0,
		  PyExc_ValueError,
		  "string length must be multiple of blocksize");
	des_ncbc_encrypt(p,q,len,
			 cipher_key, (des_cblock *)iv, dir);
    }
    else if (mode == CFB) {
	/* See special notes in the SSLeay docs on this mode... */
	des_cfb64_encrypt((unsigned char *)p,(unsigned char *)q,len,
			  cipher_key, (des_cblock *)iv, &state, dir);
    }
    else
	Py_Error(PyExc_ValueError,
		 "unsupported mode");
    
    return output;

 onError:
    return NULL;
}

PyObject *DESCipher::encrypt(PyStringObject *input)
{
    return convert(input,DES_ENCRYPT);
}

PyObject *DESCipher::decrypt(PyStringObject *input)
{
    return convert(input,DES_DECRYPT);
}

int DESCipher::isWeak()
{
    return des_is_weak_key((des_cblock *)PyString_AS_STRING(key_string));
}

/* --- DES33 Implementation ------------------------------------------- */

DES3Cipher::DES3Cipher(PyStringObject *key,
		       int cipher_mode,
		       PyStringObject *init)
    : BlockCipher(key,cipher_mode,init)
{
    Py_INCREF(key);
    key_string = key;
    
    Py_AssertWithArg(PyString_GET_SIZE(key) == keysize,
		     PyExc_ValueError,
		     "need %i byte key string for DES3",keysize);
    des_set_key((des_cblock *)PyString_AS_STRING(key),
		first_key);
    des_set_key((des_cblock *)(PyString_AS_STRING(key) + keysize/2),
		second_key);
    return;
    
 onError:
    throw mxSWIGError;
}

DES3Cipher::~DES3Cipher()
{
    Py_XDECREF(key_string);
}

PyObject *DES3Cipher::convert(PyStringObject *input,
			      int dir)
{
    PyObject *output;
    des_cblock *p = (des_cblock *)PyString_AS_STRING(input);
    des_cblock *q;
    long len = PyString_GET_SIZE(input);

    output = PyString_FromStringAndSize(NULL,PyString_GET_SIZE(input));
    if (!output)
	goto onError;
    q = (des_cblock *)PyString_AS_STRING(output);

    if (mode == ECB) {
	Py_Assert(len % blocksize == 0,
		  PyExc_ValueError,
		  "string length must be multiple of blocksize");
	while (len > 0) {
	    des_ecb2_encrypt(p,q,
			     first_key, second_key, dir);
	    p++;
	    q++;
	    len -= blocksize;
	}
    }
    else if (mode == CBC) {
	Py_Assert(len % blocksize == 0,
		  PyExc_ValueError,
		  "string length must be multiple of blocksize");
	des_ede2_cbc_encrypt(p,q,len,
			     first_key, second_key, (des_cblock *)iv, dir);
    }
    else if (mode == CFB) {
	/* See special notes in the SSLeay docs on this mode... */
	des_ede2_cfb64_encrypt((unsigned char *)p,(unsigned char *)q,len,
			       first_key, second_key,
			       (des_cblock *)iv, &state, dir);
    }
    else
	Py_Error(PyExc_ValueError,
		 "unsupported mode");
    
    return output;

 onError:
    return NULL;
}

PyObject *DES3Cipher::encrypt(PyStringObject *input)
{
    return convert(input,DES_ENCRYPT);
}

PyObject *DES3Cipher::decrypt(PyStringObject *input)
{
    return convert(input,DES_DECRYPT);
}

int DES3Cipher::isWeak()
{
    return (des_is_weak_key((des_cblock *)PyString_AS_STRING(key_string))
	    || des_is_weak_key((des_cblock *)
			       (PyString_AS_STRING(key_string) + keysize/2)));
}

/* --- CAST Implementation ------------------------------------------- */

CASTCipher::CASTCipher(PyStringObject *key,
			       int cipher_mode,
			       PyStringObject *init)
    : BlockCipher(key,cipher_mode,init)
{
    /* SSLeay imlements CAST for 5-10 byte keys with 16 rounds; the
       RFC specifies 12 rounds to be used... */
    Py_Assert(PyString_GET_SIZE(key) > 10,
	      PyExc_ValueError,
	      "key string too short: need at least 11 bytes");
    CAST_set_key(&cipher_key,
		 (int)PyString_GET_SIZE(key),
		 (unsigned char *)PyString_AS_STRING(key));
    return;
    
 onError:
    throw mxSWIGError;
}

PyObject *CASTCipher::convert(PyStringObject *input,
				  int dir)
{
    PyObject *output;
    unsigned char *p = (unsigned char *)PyString_AS_STRING(input);
    unsigned char *q;
    long len = PyString_GET_SIZE(input);
    
    output = PyString_FromStringAndSize(NULL,PyString_GET_SIZE(input));
    if (!output)
	goto onError;
    q = (unsigned char *)PyString_AS_STRING(output);

    if (mode == ECB) {
	Py_Assert(len % blocksize == 0,
		  PyExc_ValueError,
		  "string length must be multiple of blocksize");
	while (len > 0) {
	    CAST_ecb_encrypt(p,q,
			     &cipher_key, dir);
	    p += blocksize;
	    q += blocksize;
	    len -= blocksize;
	}
    }
    else if (mode == CBC) {
	Py_Assert(len % blocksize == 0,
		  PyExc_ValueError,
		  "string length must be multiple of blocksize");
	CAST_cbc_encrypt(p,q,len,
			 &cipher_key, iv, dir);
    }
    else if (mode == CFB) {
	CAST_cfb64_encrypt(p,q,len,
			   &cipher_key, iv, &state, dir);
    }
    else
	Py_Error(PyExc_ValueError,
		 "unsupported mode");
    
    return output;

 onError:
    return NULL;
}

PyObject *CASTCipher::encrypt(PyStringObject *input)
{
    return convert(input,CAST_ENCRYPT);
}

PyObject *CASTCipher::decrypt(PyStringObject *input)
{
    return convert(input,CAST_DECRYPT);
}

/* --- Hash Implementation ------------------------------------------- */

Hash::Hash(PyStringObject *data)
{
}

Hash::~Hash()
{
}

PyObject *Hash::getstate()
{
    Py_Error(PyExc_SystemError,
	     "not supported");
 onError:
    return NULL;
}

PyObject *Hash::setstate(PyStringObject *state)
{
    Py_Error(PyExc_SystemError,
	     "not supported");
 onError:
    return NULL;
}

PyObject *Hash::digest()
{
    Py_Error(PyExc_SystemError,
	     "not supported");
 onError:
    return NULL;
}

PyObject *Hash::hexdigest()
{
    PyObject *v;

    /* Digest */
    v = digest();
    if (!v)
	goto onError;
    /* Convert to HEX */
    return str2hex(v);

 onError:
    return NULL;
}

PyObject *Hash::update(PyStringObject *data)
{
    Py_Error(PyExc_SystemError,
	     "not supported");
 onError:
    return NULL;
}

/* --- MD2Hash Implementation ------------------------------------------- */

MD2Hash::MD2Hash(PyStringObject *data): Hash(data)
{
    MD2_Init(&hash_state);
    update(data);
}

PyObject *MD2Hash::getstate()
{
    PyObject *state;
    
    state = PyString_FromStringAndSize((char *)&hash_state,sizeof(hash_state));
    if (!state)
	goto onError;
    return state;

 onError:
    return NULL;
}

PyObject *MD2Hash::setstate(PyStringObject *state)
{
    Py_Assert(PyString_GET_SIZE(state) == sizeof(hash_state),
	      PyExc_ValueError,
	      "wrong state string");
    memcpy(&hash_state,PyString_AS_STRING(state),sizeof(hash_state));
    Py_ReturnNone();

 onError:
    return NULL;
}

PyObject *MD2Hash::digest()
{
    PyObject *hash;
    MD2_CTX state_copy;
    
    hash = PyString_FromStringAndSize(NULL,digestsize);
    if (!hash)
	goto onError;
    
    memcpy(&state_copy,&hash_state,sizeof(state_copy));
    MD2_Final((unsigned char *)PyString_AS_STRING(hash),
	      &hash_state);
    memcpy(&hash_state,&state_copy,sizeof(state_copy));
    return hash;

 onError:
    return NULL;
}

PyObject *MD2Hash::update(PyStringObject *data)
{
    MD2_Update(&hash_state,
	       (unsigned char *)PyString_AS_STRING(data),
	       (unsigned int)PyString_GET_SIZE(data));
    Py_ReturnNone();

 onError:
    return NULL;
}

/* --- MD5Hash Implementation ------------------------------------------- */

MD5Hash::MD5Hash(PyStringObject *data): Hash(data)
{
    MD5_Init(&hash_state);
    update(data);
}

PyObject *MD5Hash::getstate()
{
    PyObject *state;
    
    state = PyString_FromStringAndSize((char *)&hash_state,sizeof(hash_state));
    if (!state)
	goto onError;
    return state;

 onError:
    return NULL;
}

PyObject *MD5Hash::setstate(PyStringObject *state)
{
    Py_Assert(PyString_GET_SIZE(state) == sizeof(hash_state),
	      PyExc_ValueError,
	      "wrong state string");
    memcpy(&hash_state,PyString_AS_STRING(state),sizeof(hash_state));
    Py_ReturnNone();

 onError:
    return NULL;
}

PyObject *MD5Hash::digest()
{
    PyObject *hash;
    MD5_CTX state_copy;
    
    hash = PyString_FromStringAndSize(NULL,digestsize);
    if (!hash)
	goto onError;
    
    memcpy(&state_copy,&hash_state,sizeof(state_copy));
    MD5_Final((unsigned char *)PyString_AS_STRING(hash),
	      &hash_state);
    memcpy(&hash_state,&state_copy,sizeof(state_copy));
    return hash;

 onError:
    return NULL;
}

PyObject *MD5Hash::update(PyStringObject *data)
{
    MD5_Update(&hash_state,
	       (unsigned char *)PyString_AS_STRING(data),
	       (unsigned int)PyString_GET_SIZE(data));
    Py_ReturnNone();

 onError:
    return NULL;
}

/* --- SHAHash Implementation ------------------------------------------- */

SHAHash::SHAHash(PyStringObject *data): Hash(data)
{
    SHA1_Init(&hash_state);
    update(data);
}

PyObject *SHAHash::getstate()
{
    PyObject *state;
    
    state = PyString_FromStringAndSize((char *)&hash_state,sizeof(hash_state));
    if (!state)
	goto onError;
    return state;

 onError:
    return NULL;
}

PyObject *SHAHash::setstate(PyStringObject *state)
{
    Py_Assert(PyString_GET_SIZE(state) == sizeof(hash_state),
	      PyExc_ValueError,
	      "wrong state string");
    memcpy(&hash_state,PyString_AS_STRING(state),sizeof(hash_state));
    Py_ReturnNone();

 onError:
    return NULL;
}

PyObject *SHAHash::digest()
{
    PyObject *hash;
    SHA_CTX state_copy;
    
    hash = PyString_FromStringAndSize(NULL,digestsize);
    if (!hash)
	goto onError;
    
    memcpy(&state_copy,&hash_state,sizeof(state_copy));
    SHA1_Final((unsigned char *)PyString_AS_STRING(hash),
	      &hash_state);
    memcpy(&hash_state,&state_copy,sizeof(state_copy));
    return hash;

 onError:
    return NULL;
}

PyObject *SHAHash::update(PyStringObject *data)
{
    SHA1_Update(&hash_state,
	       (unsigned char *)PyString_AS_STRING(data),
	       (unsigned int)PyString_GET_SIZE(data));
    Py_ReturnNone();

 onError:
    return NULL;
}

/* --- RIPEMDHash Implementation ------------------------------------------- */

RIPEMDHash::RIPEMDHash(PyStringObject *data): Hash(data)
{
    RIPEMD160_Init(&hash_state);
    update(data);
}

PyObject *RIPEMDHash::getstate()
{
    PyObject *state;
    
    state = PyString_FromStringAndSize((char *)&hash_state,sizeof(hash_state));
    if (!state)
	goto onError;
    return state;

 onError:
    return NULL;
}

PyObject *RIPEMDHash::setstate(PyStringObject *state)
{
    Py_Assert(PyString_GET_SIZE(state) == sizeof(hash_state),
	      PyExc_ValueError,
	      "wrong state string");
    memcpy(&hash_state,PyString_AS_STRING(state),sizeof(hash_state));
    Py_ReturnNone();

 onError:
    return NULL;
}

PyObject *RIPEMDHash::digest()
{
    PyObject *hash;
    RIPEMD160_CTX state_copy;
    
    hash = PyString_FromStringAndSize(NULL,digestsize);
    if (!hash)
	goto onError;
    
    memcpy(&state_copy,&hash_state,sizeof(state_copy));
    RIPEMD160_Final((unsigned char *)PyString_AS_STRING(hash),
		    &hash_state);
    memcpy(&hash_state,&state_copy,sizeof(state_copy));
    return hash;

 onError:
    return NULL;
}

PyObject *RIPEMDHash::update(PyStringObject *data)
{
    RIPEMD160_Update(&hash_state,
		     (unsigned char *)PyString_AS_STRING(data),
		     (unsigned int)PyString_GET_SIZE(data));
    Py_ReturnNone();

 onError:
    return NULL;
}

