/* mxCrypto -- C++ Wrapper for the many fast goodies defined in SSLeay.

   (c) Marc-Andre Lemburg; All Rights Reserved; mailto: mal@lemburg.com
   See the documentation for further copyright information or contact
   the author.

 */

/* Include SSLeay parts */

#include "rc2.h"
#include "rc4.h"
#include "rc5.h"
#include "blowfish.h"
#include "idea.h"
#include "des.h"
#include "cast.h"

#include "md2.h"
#include "md5.h"
#include "sha.h"
#include "ripemd.h"

/* --- Utilities -------------------------------------------------------- */

PyObject *str2hex(PyObject *v);
PyObject *hex2str(PyObject *v);

/* --- Ciphers ---------------------------------------------------------- */

/* Modes */
#define ECB		1
#define CBC		2
#define CFB		3

/* All BlockCiphers use this block size: */
#define BLOCK_SIZE	8

class StreamCipher 
{
 protected:
    int mode;

 public:
    static int const blocksize = 1;
    static int const keysize = 0;

    StreamCipher(PyStringObject *v, int cipher_mode);
    ~StreamCipher();
    PyObject *encrypt(PyStringObject *v);
    PyObject *decrypt(PyStringObject *v);
    int getmode(void) { return mode; };
};

class BlockCipher: public StreamCipher 
{
 protected:
    int state;
    unsigned char iv[BLOCK_SIZE];

 public:
    static int const blocksize = BLOCK_SIZE;

    BlockCipher(PyStringObject *v, int cipher_mode, PyStringObject *init);
    PyObject *reinit(PyStringObject *init);
};

class RC2Cipher: public BlockCipher 
{
 private:
    RC2_KEY cipher_key;
    PyObject *convert(PyStringObject *v, int dir);

 public:
    RC2Cipher(PyStringObject *v, int cipher_mode, PyStringObject *init);
    PyObject *encrypt(PyStringObject *v);
    PyObject *decrypt(PyStringObject *v);
};

class RC4Cipher: public StreamCipher 
{

 private:
    RC4_KEY cipher_key;

 public:
    RC4Cipher(PyStringObject *v, int cipher_mode);
    PyObject *encrypt(PyStringObject *v);
    PyObject *decrypt(PyStringObject *v);
};

class RC5Cipher: public BlockCipher 
{
 private:
    RC5_32_KEY cipher_key;
    PyObject *convert(PyStringObject *v, int dir);

 public:
    RC5Cipher(PyStringObject *v, int cipher_mode, PyStringObject *init,
	      int rounds);
    PyObject *encrypt(PyStringObject *v);
    PyObject *decrypt(PyStringObject *v);
};

class BlowfishCipher: public BlockCipher 
{
 private:
    BF_KEY cipher_key;
    PyObject *convert(PyStringObject *v, int dir);

 public:
    BlowfishCipher(PyStringObject *v, int cipher_mode, PyStringObject *init);
    PyObject *encrypt(PyStringObject *v);
    PyObject *decrypt(PyStringObject *v);
};

class IDEACipher: public BlockCipher 
{
 private:
    IDEA_KEY_SCHEDULE encrypt_key;
    IDEA_KEY_SCHEDULE decrypt_key;
    PyObject *convert(PyStringObject *v, int dir);

 public:
    static int const keysize = 16;

    IDEACipher(PyStringObject *v, int cipher_mode, PyStringObject *init);
    PyObject *encrypt(PyStringObject *v);
    PyObject *decrypt(PyStringObject *v);
};

class DESCipher: public BlockCipher 
{
 private:
    PyStringObject *key_string;
    des_key_schedule cipher_key;
    PyObject *convert(PyStringObject *v, int dir);

 public:
    static int const keysize = 8;

    DESCipher(PyStringObject *v, int cipher_mode, PyStringObject *init);
    ~DESCipher();
    PyObject *encrypt(PyStringObject *v);
    PyObject *decrypt(PyStringObject *v);
    int isWeak();
};

class DES3Cipher: public BlockCipher 
{
 private:
    PyStringObject *key_string;
    des_key_schedule first_key;
    des_key_schedule second_key;
    PyObject *convert(PyStringObject *v, int dir);

 public:
    static int const keysize = 16;

    DES3Cipher(PyStringObject *v, int cipher_mode, PyStringObject *init);
    ~DES3Cipher();
    PyObject *encrypt(PyStringObject *v);
    PyObject *decrypt(PyStringObject *v);
    int isWeak();
};

class CASTCipher: public BlockCipher
{
 private:
    CAST_KEY cipher_key;
    PyObject *convert(PyStringObject *v, int dir);

 public:
    CASTCipher(PyStringObject *v, int cipher_mode, PyStringObject *init);
    PyObject *encrypt(PyStringObject *v);
    PyObject *decrypt(PyStringObject *v);
};

/* --- Hashes ----------------------------------------------------------- */

class Hash 
{
 public:
    static int const digestsize = 16;
    
    Hash(PyStringObject *data);
    virtual ~Hash();
    PyObject *update(PyStringObject *data);
    PyObject *getstate();
    PyObject *setstate(PyStringObject *state);
    virtual PyObject *digest();
    PyObject *hexdigest();
};

class MD2Hash: public Hash
{
 private:
    MD2_CTX hash_state;
    
 public:
    static int const digestsize = MD2_DIGEST_LENGTH;
    
    MD2Hash(PyStringObject *v);
    PyObject *update(PyStringObject *data);
    PyObject *getstate();
    PyObject *setstate(PyStringObject *state);
    virtual PyObject *digest();
};

class MD5Hash: public Hash
{
 private:
    MD5_CTX hash_state;
    
 public:
    static int const digestsize = MD5_DIGEST_LENGTH;
    
    MD5Hash(PyStringObject *v);
    PyObject *update(PyStringObject *data);
    PyObject *getstate();
    PyObject *setstate(PyStringObject *state);
    virtual PyObject *digest();
};

class SHAHash: public Hash
{
 private:
    SHA_CTX hash_state;
    
 public:
    static int const digestsize = SHA_DIGEST_LENGTH;
    
    SHAHash(PyStringObject *v);
    PyObject *update(PyStringObject *data);
    PyObject *getstate();
    PyObject *setstate(PyStringObject *state);
    virtual PyObject *digest();
};

class RIPEMDHash: public Hash
{
 private:
    RIPEMD160_CTX hash_state;
    
 public:
    static int const digestsize = RIPEMD160_DIGEST_LENGTH;
    
    RIPEMDHash(PyStringObject *v);
    PyObject *update(PyStringObject *data);
    PyObject *getstate();
    PyObject *setstate(PyStringObject *state);
    virtual PyObject *digest();
};
